subroutine wp2ay1(appr, lmatra, lmasse, lamor, sigma,&
                  lbloq, yh, yb, zh, zb,&
                  u1, u2, u3, v, n,&
                  solveu)
    implicit none
#include "jeveux.h"
#include "asterfort/assert.h"
#include "asterfort/dismoi.h"
#include "asterfort/jelira.h"
#include "asterfort/mrmult.h"
#include "asterfort/resoud.h"
!
    character(len=1) :: appr
    complex(kind=8) :: v(*), sigma
    real(kind=8) :: u1(*), u2(*), u3(*), yh(*), yb(*), zh(*), zb(*)
    integer :: lmatra, lmasse, lamor, n, lbloq(*)
    character(len=19) :: solveu
!     ------------------------------------------------------------------
! ======================================================================
! COPYRIGHT (C) 1991 - 2015  EDF R&D                  WWW.CODE-ASTER.ORG
! THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
! IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
! THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
! (AT YOUR OPTION) ANY LATER VERSION.
!
! THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
! WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
! MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
! GENERAL PUBLIC LICENSE FOR MORE DETAILS.
!
! YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
! ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
!   1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
! ======================================================================
! ======================================================================
!                   T               T
!     CALCUL (ZH,ZB)   = A * (YH,YB)
!
!     OU A EST L' OPERATEUR (REEL) DONT ON CHERCHE UNE APPROXIMATION
!     DES ELEMENTS PROPRES
!     ------------------------------------------------------------------
! IN  APPR   : K : INDICATEUR D' APPROCHE ('R'/'I') POUR A
! IN  LMATRA : I : FACTORISEE LDLT (DANS C) DE LA MATRICE DYNAMIQUE
! IN  LMASSE : I : MATRICE DE MASSE
! IN  LAMOR  : I : MATRICE D' AMORTISSEMENT
! IN  SIGMA  : C : VALEUR DU SHIFT
! IN  YH     : R : PARTIE SUPERIEUR DE Y
! IN  YB     : R : PARTIE INFERIEURE DE Y
! IN  N      : I : DIMENSION DE MATRICE
! IN  LBLOQ  : I : TYPE DES DDL (LBOLOQ(I) = 0 <=> DDL(I) = BLOQUE)
! OUT ZH     : R : PARTIE SUPERIEURE DU RESULTAT
! OUT ZB     : R : PARTIE INFERIEURE DU RESULTAT
! VAR U1     : R : VECTEUR DE TRAVAIL, EN SORTIE VAUT AMOR *YH
! VAR U2     : R : VECTEUR DE TRAVAIL, EN SORTIE VAUT MASSE*YB
! VAR U3     : R : VECTEUR DE TRAVAIL, EN SORTIE VAUT MASSE*YH
! VAR U4     : R : VECTEUR DE TRAVAIL
! VAR V      : R : VECTEUR DE TRAVAIL
! IN  SOLVEU : K19: SD SOLVEUR POUR PARAMETRER LE SOLVEUR LINEAIRE
!     ------------------------------------------------------------------
!
!
    real(kind=8) :: zero, sr, si
    integer :: i
    complex(kind=8) :: cbid
    character(len=1) :: kbid, rouc
    character(len=14) :: nu
    character(len=19) :: k19bid, matass, chcine, criter
    integer :: iret
    cbid = dcmplx(0.d0, 0.d0)
!     ------------------------------------------------------------------
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
    zero = 0.0d0
    sr = dble(sigma)
    si = dimag(sigma)
!
! INIT. OBJETS ASTER
    matass=zk24(zi(lmatra+1))
    chcine=' '
    criter=' '
    k19bid=' '
    call mrmult('ZERO', lamor, yh, u1, 1, .false._1)
    call mrmult('ZERO', lmasse, yb, u2, 1, .false._1)
    call mrmult('ZERO', lmasse, yh, u3, 1, .false._1)
!-RM-DEB
!     LA BOUCLE 5 REALISE LE PRODUIT PAR MASSE*INV(MASSE_REG)*MASSR
!     OR CETTE MATRICE EST EGALE A MASSE
!---> VOIR CE QUI SE PASSE QUAND LA BOUCLE EST SUPPRIMEE
    do i = 1, n, 1
        u3(i) = u3(i)*lbloq(i)
        u2(i) = u2(i)*lbloq(i)
    end do
!-RM-FIN
    call dismoi('NOM_NUME_DDL', matass, 'MATR_ASSE', repk=nu)
    call jelira(matass//'.VALM', 'TYPE', cval=rouc)
    if (rouc.eq.'C') then
        do i = 1, n, 1
            v(i) = dcmplx(u1(i)) + sigma*dcmplx(u3(i)) + dcmplx(u2(i))
        end do
        call resoud(matass, k19bid, solveu, chcine, 1,&
                    k19bid, k19bid, kbid, [0.d0], v,&
                    criter, .false._1, 0, iret)
        if (appr .eq. 'R') then
            do i = 1, n, 1
                zh(i) = - dble(v(i))
                zb(i) = (yh(i) - dble(sigma*v(i)))*lbloq(i)
            end do
        else
            do i = 1, n, 1
                zh(i) = - dimag(v(i))
                zb(i) = - dimag(sigma*v(i))*lbloq(i)
            end do
        endif
    else if ((si .eq. zero).and.(rouc.eq.'R')) then
        do i = 1, n, 1
            u1(i) = u1(i) + sr*u3(i) + u2(i)
        end do
        call resoud(matass, k19bid, solveu, chcine, 1,&
                    k19bid, k19bid, kbid, u1, [cbid],&
                    criter, .false._1, 0, iret)
        do i = 1, n, 1
            zh(i) = -u1(i)
            zb(i) = (yh(i) - sr*u1(i))*lbloq(i)
        end do
    else
! Cas non prevu, cf wpfopr/wpsorn
        ASSERT(.false.)
    endif
end subroutine
