subroutine nmrenu(modelz     , list_func_acti, list_load, solver, sdcont_defi,&
                  sdcont_solv, nume_ddl      , l_renumber)
!
implicit none
!
#include "asterf_types.h"
#include "asterfort/cfdisl.h"
#include "asterfort/infdbg.h"
#include "asterfort/isfonc.h"
#include "asterfort/jedema.h"
#include "asterfort/jemarq.h"
#include "asterfort/jeveuo.h"
#include "asterfort/numer3.h"
!
! ======================================================================
! COPYRIGHT (C) 1991 - 2015  EDF R&D                  WWW.CODE-ASTER.ORG
! THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
! IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
! THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
! (AT YOUR OPTION) ANY LATER VERSION.
!
! THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
! WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
! MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
! GENERAL PUBLIC LICENSE FOR MORE DETAILS.
!
! YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
! ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
!   1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
! ======================================================================
! person_in_charge: mickael.abbas at edf.fr
!
    character(len=*), intent(in) :: modelz
    character(len=24), intent(inout) :: nume_ddl
    character(len=19), intent(in) :: list_load
    character(len=19), intent(in) :: solver
    character(len=24), intent(in) :: sdcont_defi
    character(len=24), intent(in) :: sdcont_solv
    integer, intent(in) :: list_func_acti(*)
    aster_logical, intent(out) :: l_renumber
!
! --------------------------------------------------------------------------------------------------
!
! Non-linear algorithm
!
! Renumbering equations ?
!
! --------------------------------------------------------------------------------------------------
!
! IO  nume_ddl       : name of numbering object (NUME_DDL)
! In  solver         : name of solver datastructure
! In  model          : name of model datastructure
! In  list_load      : list of loads
! In  sdcont_defi    : name of contact definition datastructure (from DEFI_CONTACT)
! In  sdcont_solv    : name of contact solving datastructure
! In  list_func_acti : list of active functionnalities
! Out l_renumber     : .true. if renumber
!
! --------------------------------------------------------------------------------------------------
!
    integer :: ifm, niv
    aster_logical :: l_cont, l_cont_cont, l_cont_xfem, l_cont_elem, l_cont_xfem_gg
    character(len=24) :: sd_iden_rela
    character(len=24) :: crnudd
    aster_logical, pointer :: v_crnudd(:) => null()
    character(len=24) :: nosdco
    character(len=24), pointer :: v_nosdco(:) => null()
!
! --------------------------------------------------------------------------------------------------
!
    call infdbg('MECA_NON_LINE', ifm, niv)
!
! - Initializations
!
    crnudd       = sdcont_solv(1:14)//'.NUDD'
    nosdco       = sdcont_solv(1:14)//'.NOSDCO'
    l_renumber   = .false.
    l_cont       = isfonc(list_func_acti,'CONTACT')
    if (.not.l_cont) then
        goto 999
    endif
!
! - Get identity relation datastructure
!
    call jeveuo(nosdco, 'L', vk24 = v_nosdco)
    sd_iden_rela = v_nosdco(4)
!
! - Contact method
!
    l_cont_elem = isfonc(list_func_acti,'ELT_CONTACT')
    l_cont_xfem = isfonc(list_func_acti,'CONT_XFEM')
    l_cont_cont = isfonc(list_func_acti,'CONT_CONTINU')
!
! - Numbering to change ?
!
    if (l_cont_elem) then
        if (l_cont_xfem) then
            l_cont_xfem_gg = cfdisl(sdcont_defi,'CONT_XFEM_GG')
            if (l_cont_xfem_gg) then
               l_renumber = .true.
            else
               l_renumber = .false.
            endif
        else
            call jeveuo(crnudd, 'E', vl = v_crnudd)
            l_renumber = v_crnudd(1)
            v_crnudd(1) = .false.
        endif
    endif
!
! - Re-numbering
!
    if (l_renumber) then
        if (niv .ge. 2) then
            write (ifm,*) '<MECANONLINE> ...... RE-CREATION DU NUME_DDL '
        endif
        call numer3(modelz, list_load, solver, nume_ddl, sd_iden_rela)
    endif
!
999 continue
!
end subroutine
